@@ -18,6 +18,7 @@ namedList <- function (...) {
 
 ## Sparse Schur complement (Marginal of precision matrix)
 ##' @importFrom Matrix Cholesky solve
+##' @importFrom methods as
 GMRFmarginal <- function(Q, i, ...) {
     ind <- seq_len(nrow(Q))
     i1 <- (ind)[i]
@@ -49,17 +50,30 @@ get_matdim <- function(ntri) {
     as.integer(round(0.5 * (1 + sqrt(1 + 8 * ntri))))
 }
 
-##' translate vector of correlation parameters to correlation values
+##' transform correlation parameters to and from glmmTMB parameterization
 ##' @param theta vector of internal correlation parameters (elements of scaled Cholesky factor, in \emph{row-major} order)
 ##' @param return_val return a vector of correlation values from the lower triangle ("vec"), or the full correlation matrix ("mat")? 
 ##' @return a vector of correlation values (\code{get_cor}) or glmmTMB scaled-correlation parameters (\code{put_cor})
-##' @details These functions follow the definition at \url{http://kaskr.github.io/adcomp/classdensity_1_1UNSTRUCTURED__CORR__t.html}:
-##' if \eqn{L} is the lower-triangular matrix with 1 on the diagonal and the correlation parameters in the lower triangle, then the correlation matrix is defined as \eqn{\Sigma = D^{-1/2} L L^\top D^{-1/2}}{Sigma = sqrt(D) L L' sqrt(D)}, where \eqn{D = \textrm{diag}(L L^\top)}{D = diag(L L')}. For a single correlation parameter \eqn{\theta_0}{theta0}, this works out to \eqn{\rho = \theta_0/\sqrt{1+\theta_0^2}}{rho = theta0/sqrt(1+theta0^2)}. The \code{get_cor} function returns the elements of the lower triangle of the correlation matrix, in column-major order.
+##' @details
+##' \code{\link{get_cor}} transforms from the glmmTMB parameterization (components of a \code{theta} parameter vector) to correlations;
+##' \code{\link{put_cor}} does the inverse transformations, from correlations to \code{theta} values.
+##' 
+##' These functions follow the definition at \url{http://kaskr.github.io/adcomp/classdensity_1_1UNSTRUCTURED__CORR__t.html}:
+##' if \eqn{L} is the lower-triangular matrix with 1 on the diagonal and the correlation parameters in the lower triangle, then the correlation matrix is defined as \eqn{\Sigma = D^{-1/2} L L^\top D^{-1/2}}{Sigma = sqrt(D) L L' sqrt(D)}, where \eqn{D = \textrm{diag}(L L^\top)}{D = diag(L L')}. For a single correlation parameter \eqn{\theta_0}{theta0} (i.e. the correlation in a 2x2 correlation matrix), this works out to \eqn{\rho = \theta_0/\sqrt{1+\theta_0^2}}{rho = theta0/sqrt(1+theta0^2)}. The \code{get_cor} function returns the elements of the lower triangle of the correlation matrix, in column-major order.
+##'
+##' These functions also work for AR1 correlation parameters.
 ##' @examples
 ##' th0 <- 0.5
-##' stopifnot(all.equal(get_cor(th0),th0/sqrt(1+th0^2)))
+##' stopifnot(all.equal(get_cor(th0), th0/sqrt(1+th0^2)))
 ##' set.seed(101)
-##' C <- get_cor(rnorm(21), return_val = "mat")
+##' ## pick 6 values for a random 4x4 correlation matrix
+##' print(C <- get_cor(rnorm(6), return_val = "mat"), digits = 3)
+##' ## transform a correlation matrix to a theta vector
+##' cor_mat <- matrix(c(1,0.3,0.1,
+##'                     0.3,1,0.2,
+##'                     0.1,0.2,1), ncol = 3)
+##' put_cor(cor_mat, "mat")
+##' put_cor(cor_mat[lower.tri(cor_mat)], "vec")
 ##' ## test: round-trip
 ##' stopifnot(all.equal(get_cor(put_cor(C), return_val = "mat"), C))
 ##' @export
@@ -345,104 +359,120 @@ up2date <- function(oldfit, update_gauss_disp = FALSE) {
   openmp(1)  ## non-parallel/make sure NOT grabbing all the threads!
   obj <- oldfit$obj
   ee <- obj$env
+  ee2 <- oldfit$sdr$env
 
   if (isNullPointer(oldfit$obj$env$ADFun$ptr)) {
 
-      pars <- c(grep("last\\.par", names(ee), value = TRUE), "par",
-                "parfull")
+    pars <- c(grep("last\\.par", names(ee), value = TRUE), "par",
+              "parfull")
 
-      ## using ee$parList() rather than ee$parameters should help
-      ##  with mapped parameter s... ??
-      params <- ee$parList()
+    ## using ee$parList() rather than ee$parameters should help
+    ##  with mapped parameters... ??
+    params <- ee$parList()
 
-      if (length(ee$map) > 0) {
-          for (n in names(ee$map)) {
-              ee$parameters[[n]] <- params[[n]]
-          }
+    if (length(ee$map) > 0) {
+      for (n in names(ee$map)) {
+        ee$parameters[[n]] <- params[[n]]
       }
-      
-      ## change name of thetaf to psi
-      if ("thetaf" %in% names(params)) {
-          ee$parameters$psi <- params$thetaf
-          ee$parameters$thetaf <- NULL
-          pars <- c(grep("last\\.par", names(ee), value = TRUE),
-                    "par")
-          for (p in pars) {
-              if (!is.null(nm <- names(ee[[p]]))) {
-                  names(ee[[p]])[nm == "thetaf"] <- "psi"
-              }
-          }
+    }
+
+    ## all elements containing parameter values
+    par_els <- c(grep("last\\.par", names(ee), value = TRUE), "par")
+
+    ## rename in place rather than assigning/NULLing
+    rename_fun <- function(x, oldname, newname) {
+      nm <- names(x)
+      names(x)[nm == oldname] <- newname
+      x
+    }
+
+    ## might be able to repeat *slightly* less ...
+    
+    ## change name of thetaf to psi
+    if ("thetaf" %in% names(params)) {
+      ee$parameters <- rename_fun(ee$parameters, "thetaf", "psi")
+      ee2$parameters <- rename_fun(ee2$parameters, "thetaf", "psi")
+      oldfit$fit$par <- rename_fun(oldfit$fit$par, "thetaf", "psi")
+      oldfit$fit$parfull <- rename_fun(oldfit$fit$parfull, "thetaf", "psi")
+
+      for (p in par_els) {
+        if (!is.null(nm <- names(ee[[p]]))) {
+          ee[[p]] <- rename_fun(ee[[p]], "thetaf", "psi")
+        }
       }
-      if ("betad" %in% names(ee$parameters)) { #FIXME: DRY
-      	ee$parameters$betadisp <- params$betad
-      	ee$parameters$betad <- NULL
-      	pars <- c(grep("last\\.par", names(ee), value = TRUE),
-      						"par")
-      	for (p in pars) {
-      		if (!is.null(nm <- names(ee[[p]]))) {
-      			names(ee[[p]])[nm == "betad"] <- "betadisp"
+    }
+    
+    if ("betad" %in% names(ee$parameters)) {
+      ee$parameters <- rename_fun(ee$parameters, "betad", "betadisp")
+      ee2$parameters <- rename_fun(ee2$parameters, "betad", "betadisp")
+      oldfit$fit$par <- rename_fun(oldfit$fit$par, "betad", "betadisp")
+      oldfit$fit$parfull <- rename_fun(oldfit$fit$parfull, "betad", "betadisp")
+      for (p in par_els) {
+        if (!is.null(nm <- names(ee[[p]]))) {
+          ee[[p]] <- rename_fun(ee[[p]], "betad", "betadisp")
       		}
-      	}
-      	ee$data$Xdisp <- ee$data$Xd
-      	ee$data$Xd <- NULL
-      	ee$data$dispoffset <- ee$data$doffset
-      	ee$data$doffset <- NULL
-      }
-      if(!"Zdisp" %in% names(ee$data)) {
-      	ee$data$Zdisp <- new("dgTMatrix",Dim=c(as.integer(nrow(ee$data$Xdisp)),0L)) ## matrix(0, ncol=0, nrow=nobs)
-      	ee$parameters$bdisp <- rep(0, ncol(ee$data$Zdisp))
-      	ee$parameters$thetadisp <- numeric(0)
-      }
-      ee2 <- oldfit$sdr$env
-      if ("thetaf" %in% names(ee2$parameters)) {
-          ee2$parameters$psi <- ee2$parameters$thetaf
-          ee2$parameters$thetaf <- NULL
       }
+      ee$data <- rename_fun(ee$data, "Xd", "Xdisp")
+      ee$data <- rename_fun(ee$data, "doffset", "dispoffset")
+    }
+    if(!"Zdisp" %in% names(ee$data)) {
+      ee$data$Zdisp <- new("dgTMatrix",Dim=c(as.integer(nrow(ee$data$Xdisp)),0L)) ## matrix(0, ncol=0, nrow=nobs)
+      ee$parameters$bdisp <- rep(0, ncol(ee$data$Zdisp))
+      ee$parameters$thetadisp <- numeric(0)
+    }
+    if (!"aggregate" %in% names(ee$data)) {
+      ee$data[["aggregate"]] <- numeric(0)
+    }
 
-      for (i in seq_along(ee$data$terms)) {
-          ee$data$terms[[i]]$simCode <- .valid_simcode[["random"]]
-      }
-      for (i in seq_along(ee$data$termszi)) {
-          ee$data$termszi[[i]]$simCode <- .valid_simcode[["random"]]
+    for (comp in c("terms", "termszi", "termsdisp")) {
+      for (i in seq_along(ee$data[[comp]])) {
+        ee$data[[comp]][[i]]$simCode <- .valid_simcode[["random"]]
+        ee$data[[comp]][[i]]$fullCor <- 1.0
       }
+    }
       
-      if ("betad" %in% names(ee2$parameters)) { #FIXME: DRY
-      	ee2$parameters$betadisp <- ee2$parameters$betad
-      	ee2$parameters$betad <- NULL
+
+    ## prior_ivars, prior_fvars are defined in priors.R
+    if (!"prior_distrib" %in% names(ee$data)) {
+      ## these are DATA_IVECTOR but apparently after processing
+      ##  TMB turns these into numeric ... ??
+      for (v in prior_ivars) ee$data[[v]] <- numeric(0)
+      for (v in prior_fvars) ee$data[[v]] <- numeric(0)
+      
+    }
+
+    ## switch from variance to SD parameterization
+    if (update_gauss_disp &&
+        family(oldfit)$family == "gaussian") {
+      ee$parameters$betadisp <- ee$parameters$betadisp/2
+      for (p in par_els) {
+        if (!is.null(nm <- names(ee[[p]]))) {
+          ee[[p]][nm == "betadisp"] <- ee[[p]][nm == "betadisp"]/2
+        }
+        if (!is.null(nm <- names(oldfit$fit[[p]]))) {
+          oldfit$fit[[p]][nm == "betadisp"] <- oldfit$fit[[p]][nm == "betadisp"]/2
+        }
       }
+    }
 
-      ## prior_ivars, prior_fvars are defined in priors.R
-      if (!"prior_distrib" %in% names(ee$data)) {
-          ## these are DATA_IVECTOR but apparently after processing
-          ##  TMB turns these into numeric ... ??
-          for (v in prior_ivars) ee$data[[v]] <- numeric(0)
-          for (v in prior_fvars) ee$data[[v]] <- numeric(0)
-
-      }
-
-      ## switch from variance to SD parameterization
-      if (update_gauss_disp &&
-          family(oldfit)$family == "gaussian") {
-          ee$parameters$betadisp <- params$betadisp/2
-          for (p in pars) {
-              if (!is.null(nm <- names(ee[[p]]))) {
-                  ee[[p]][nm == "betadisp"] <- ee[[p]][nm == "betadisp"]/2
-              }
-              if (!is.null(nm <- names(oldfit$fit[[p]]))) {
-                  oldfit$fit[[p]][nm == "betadisp"] <- oldfit$fit[[p]][nm == "betadisp"]/2
-              }
-          }
-      }
-
-      oldfit$obj <- with(ee,
+    oldfit$obj <- with(ee,
                        TMB::MakeADFun(data,
                                       parameters,
                                       map = map,
                                       random = random,
                                       silent = silent,
                                       DLL = "glmmTMB"))
-      oldfit$obj$env$last.par.best <- ee$last.par.best
-      ##
+
+    ## replace carefully (ordering problem)
+    replace_vals <- function(x, y) {
+      for (nm in unique(names(x))) {
+        x[names(x)==nm] <- y[names(y)==nm]
+      }
+      x
+    }
+
+    oldfit$obj$env$last.par.best <- replace_vals(oldfit$obj$env$last.par.best, ee$last.par.best)
+
   }
 
   for (t in c("condReStruc", "ziRestruc", "dispRestruc")) {
@@ -471,7 +501,10 @@ up2date <- function(oldfit, update_gauss_disp = FALSE) {
       ## n.b. can't use ...$priors <- NULL
       oldfit$modelInfo["priors"] <- list(NULL)
   }
-
+  if (!"aggregate" %in% names(ee$data)) {
+  	ee$data[["aggregate"]] <- numeric(0)
+  }
+  
   if ("Xd" %in% names(ee$data)) {
       ee$data[["Xdisp"]] <- ee$data[["Xd"]]
       ee$data[["Xd"]] <- NULL
@@ -489,15 +522,11 @@ up2date <- function(oldfit, update_gauss_disp = FALSE) {
 #'
 #' @param fn partial path to system file (e.g. test_data/foo.rda)
 #' @param verbose print names of updated objects?
-#' @param mustWork fail if file not found?
 #' @param \dots values passed through to \code{up2date}
 #' @export
-gt_load <- function(fn, verbose=FALSE, mustWork = FALSE, ...) {
+gt_load <- function(fn, verbose=FALSE, ...) {
     sf <- system.file(fn, package = "glmmTMB")
-    found_file <- file.exists(sf)
-    if (mustWork && !found_file) {
-        stop("couldn't find system file ", sf)
-    }
+    if (!file.exists(sf)) return(NULL)
 
     L <- load(sf)
     for (m in L) {
@@ -507,7 +536,7 @@ gt_load <- function(fn, verbose=FALSE, mustWork = FALSE, ...) {
         }
         assign(m, get(m), parent.env(), envir = parent.frame())
     }
-    return(found_file)
+    return(L)
 }
 
 #' truncated distributions
@@ -559,7 +588,8 @@ dtruncated_nbinom1 <- function(x, phi, mu, k=0, log=FALSE) {
 par_components <- c("beta","betazi","betadisp","theta","thetazi","psi")
 
 ## all parameters, including both mapped and rank-dropped
-getParnames <- function(object, full, include_dropped = TRUE, include_mapped = TRUE) {
+getParnames <- function(object, full, include_dropped = TRUE, include_mapped = TRUE,
+                        include_mapequal = FALSE) {
                            
   mkNames <- function(tag="") {
       X <- getME(object,paste0("X",tag))
@@ -575,6 +605,7 @@ getParnames <- function(object, full, include_dropped = TRUE, include_mapped = T
           nn[ dropped] <- names(dropped)
       }
       if (trivialFixef(nn, tag)
+          && !(tag == "disp" && (!trivialDisp(object) && !zeroDisp(object)))
           ## if 'full', keep disp even if trivial, if used by family
           && !(full && tag =="disp" &&
                (usesDispersion(family(object)$family) && !zeroDisp(object)))) {
@@ -618,12 +649,17 @@ getParnames <- function(object, full, include_dropped = TRUE, include_mapped = T
      map <- object$obj$env$map
      if (length(map)>0) {
          for (m in seq_along(map)) {
-            if (length(NAmap <- which(is.na(map[[m]])))>0) {
+             if (!include_mapequal) {
+                 missmap <- which(is.na(map[[m]]) | duplicated(map[[m]]))
+             } else {
+                 missmap <- which(is.na(map[[m]]))
+             }
+             if (length(missmap)>0) {
                 w <- match(names(map)[m], par_components) ##
                 if (length(nameList)>=w) { ## may not exist if !full
-                    nameList[[w]] <- nameList[[w]][-NAmap]
+                    nameList[[w]] <- nameList[[w]][-missmap]
                 }
-            }
+            }  ## if NA/duplicated values
          } ## for (m in seq_along(map))
      } ## if (length(map) > 0)
   }
@@ -745,38 +781,50 @@ set_simcodes <- function(g, val = "zero", terms = "ALL") {
 ##' @param newparams a list of parameters containing sub-vectors
 ##' (\code{beta}, \code{betazi}, \code{betadisp}, \code{theta}, etc.) to
 ##' be used in the model. If \code{b} is specified in this list, then the conditional modes/BLUPs
-##' will be set to these values; otherwise they will be drawn from the appropriate Normal distribution
+##' will be set to these values; otherwise they will be drawn from the appropriate Normal distribution.
+##' See \code{vignette("covstruct", package = "glmmTMB")} for details on the parameterizations used
+##' for various random-effects models (i.e., \code{theta}).
 ##' @param ... other arguments to \code{glmmTMB} (e.g. \code{family})
 ##' @param return_val what information to return: "sim" (the default) returns a list of vectors of simulated outcomes; "pars" returns the default parameter vector (this variant does not require \code{newparams} to be specified, and is useful for figuring out the appropriate dimensions of the different parameter vectors); "object" returns a fake \code{glmmTMB} object (useful, e.g., for retrieving the Z matrix (\code{getME(simulate_new(...), "Z")}) or covariance matrices (\code{VarCorr(simulate_new(...))}) implied by a particular set of input data and parameter values)
+##' @details Use the \code{weights} argument to set the size/number of trials per observation for binomial-type models; the default is 1 for every observation (i.e., Bernoulli trials)
+##' @seealso \code{\link{glmmTMB}}, \code{\link{family_glmmTMB}} (for conditional distribution parameterizations [\code{betadisp}]), \code{\link{put_cor}} (for correlation matrix parameterizations)
 ##' @examples
-##' ## use Salamanders data for structure/covariates
+##' ## use Salamanders data for observational design and covariate values
+##' ## parameters used here are sensible, but do not fit the original data
+##' params <- list(beta = c(2, 1),
+##'                betazi = c(-0.5, 0.5), ## logit-linear model for zi
+##'                betadisp = log(2), ## log(NB dispersion)
+##'                theta = log(1)) ## log(among-site SD)
 ##' sim_count <- simulate_new(~ mined + (1|site),
 ##'              newdata = Salamanders,
 ##'              zi = ~ mined,
 ##'              family = nbinom2,
-##'              newparams = list(beta = c(2, 1),
-##'                          betazi = c(-0.5, 0.5), ## logit-linear model for zi
-##'                          betadisp = log(2), ## log(NB dispersion)
-##'                          theta = log(1)) ## log(among-site SD)
+##'              seed = 101,
+##'              newparams = params
 ##' )
+##' ## simulate_new with return="sim" always returns a list of response vectors
+##' Salamanders$sim_count <- sim_count[[1]]    
+##' summary(glmmTMB(sim_count ~ mined + (1|site), data=Salamanders, ziformula=~mined, family=nbinom2))
+##' ## return a glmmTMB object
 ##' sim_obj <- simulate_new(~ mined + (1|site),
 ##'             return_val = "object",
 ##'              newdata = Salamanders,
 ##'              zi = ~ mined,
 ##'              family = nbinom2,
-##'              newparams = list(beta = c(2, 1),
-##'                          betazi = c(-0.5, 0.5), ## logit-linear model for zi
-##'                          betad = log(2), ## log(NB dispersion)
-##'                          theta = log(1)) ## log(among-site SD)
-##' )
+##'              newparams = params)
+##' ## simulate Gaussian data, multivariate random effect
 ##' data("sleepstudy", package = "lme4")
 ##' sim_obj <- simulate_new(~ 1 + (1|Subject) + ar1(0 + factor(Days)|Subject),
 ##'              return_val = "pars",
 ##'              newdata = sleepstudy,
 ##'              family = gaussian,
 ##'              newparams = list(beta = c(280, 1),
-##'                          betad = log(2), ## log(SD)
-##'                          theta = log(c(2, 2, 1))),
+##'                          betad = log(2), ## log(residual std err)
+##'                          theta = c(log(2), ## log(SD(subject))
+##'                                    log(2), ## log(SD(slope))
+##'                                    ## AR1 correlation = 0.2
+##'                                    put_cor(0.2, input_val = "vec"))
+##'                          )
 ##' )
 ##' 
 ##' @export
@@ -784,10 +832,11 @@ simulate_new <- function(object,
                          nsim = 1,
                          seed = NULL,
                          family = gaussian,
-                         newdata, newparams, ...,
+                         newdata,
+                         newparams = NULL, ...,
                          return_val = c("sim", "pars", "object")) {
     return_val <- match.arg(return_val)
-    family <- get_family(family)
+    family <- get_family(family, deparse(substitute(family)))
     ## truncate
     if (length(object) == 3) stop("simulate_new should take a one-sided formula")
     newparams0 <- newparams
@@ -917,7 +966,7 @@ match_names <- function(x, to_parvec = FALSE, prefix = "beta") {
     }
 }
 
-get_family <- function(family) {
+get_family <- function(family, f_name) {
     if (is.character(family)) {
         if (family=="beta") {
             family <- "beta_family"
@@ -934,9 +983,8 @@ get_family <- function(family) {
 
     ## FIXME: what is this doing? call to a function that's not really
     ##  a family creation function?
-    if (is.null(family$family)) {
-      print(family)
-      stop("after evaluation, 'family' must have a '$family' element")
+    if (isS4(family) || is.null(family$family)) {
+        stop(sprintf("after evaluation, 'family' (specified as '%s') must be a list containing a '$family' element", f_name))
     }
     return(family)
 }
@@ -1003,3 +1051,7 @@ our_binom_initialize <- function(family) {
     return(b0)
 }
 
+#' retrieve current value of TMB autopar setting
+get_autopar <- function() {
+    attr(TMB::openmp(DLL = "glmmTMB"), "autopar")
+}
