@@ -21,22 +21,31 @@ stripTMBVC <- function(x) {
     r <- VarCorr(x)[["cond"]]
     for (i in seq_along(r)) {
         attr(r[[i]],"blockCode") <- NULL
+        class(r[[i]]) <- c("matrix", "array")
     }
     return(r)
 }
 
+## check numerical equality, ignoring attributes *and* classes
+expect_all_equal_strip <- function(x, y, tolerance = 1e-8) {
+  expect_true(all.equal(x, y, check.attributes = FALSE, check.class = FALSE, tolerance = tolerance))
+}
+
 test_that("basic glmer vs glmmTMB", {
-    expect_equal(stripTMBVC(fm1),unclass(VarCorr(fm1C)),
-                 tol=2e-3)
-    expect_equal(stripTMBVC(gm1),unclass(VarCorr(gm1C)),
-                 tol=5e-3)
+    expect_all_equal_strip(stripTMBVC(fm1), unclass(VarCorr(fm1C)),
+                          tolerance=2e-3)
+    expect_all_equal_strip(stripTMBVC(gm1), unclass(VarCorr(gm1C)),
+                          tolerance=5e-3)
 })
 
 ## have to take only last 4 lines
 ## some white space diffs introduced in fancy-corr-printing
+## FIXME: need to match formatting better?
 pfun <- function(x) squash_white(capture.output(print(VarCorr(x),digits=2)))
-expect_equal(tail(pfun(fm1),4),
-             pfun(fm1C))
+test_that("VarCorr output equivalent to lme4", {
+    expect_equal(tail(pfun(fm1),4),
+                 pfun(fm1C))
+})
 
 data("Pixel", package="nlme")
 ## nPix <- nrow(Pixel)
@@ -230,3 +239,11 @@ test_that("VarCorr for models with RE in dispersion", {
                  tolerance = 1e-5)
 })
 
+## need to fix diag, comp symm ...
+skip_models <- character(0)
+models <- setdiff(gt_load("test_data/models.rda"), skip_models)
+for (m in models) {
+    obj <- get(m)
+    if (!inherits(obj, "glmmTMB")) next
+    capture.output(print(VarCorr(obj)))
+}
